import os
from datasets import load_dataset,DatasetDict
from transformers import AutoTokenizer
from transformers import LlamaTokenizerFast

def calc_tokens(dataset_path:str, dataset_split:float, token_path:str,epoch:int):
    try:
        tokenizer = AutoTokenizer.from_pretrained(token_path, trust_remote_code=True)
    except Exception as e:
        tokenizer = LlamaTokenizerFast.from_pretrained(token_path)
    file_list = [
        os.path.join(dataset_path, filename)
        for filename in os.listdir(dataset_path)
        if filename.endswith((".json", ".jsonl"))
    ]
    hf_dataset = load_dataset(path='json', data_files=file_list)
    if dataset_split > 0:
        hf_dataset = next(iter(hf_dataset.values())) if isinstance(hf_dataset,DatasetDict) else hf_dataset
        dataset_len = len(hf_dataset)
        array = [i for i in range(dataset_len+1)]
        percent_count = int(max(dataset_len * dataset_split, 1))
        cut_off_len = 200 if percent_count > 200 else percent_count
        hf_dataset = hf_dataset.select(array[:-cut_off_len])

    def count_tokens(example):
        len_token_cnts = []
        for instruction,input,output in zip(example['instruction'],example['input'],example['output']):
            full_prompt = instruction + input + output
            token_cnt = tokenizer.tokenize(full_prompt)
            len_token_cnt = len(token_cnt)
            len_token_cnts.append(len_token_cnt)
        return {'total_tokens':len_token_cnts}
    
    token_dict = hf_dataset.map(count_tokens,
                                remove_columns=['instruction', 'input', 'output'],
                                batched=True)

    col_token = token_dict['train']['total_tokens'] if isinstance(hf_dataset, DatasetDict) else token_dict['total_tokens'] 
    print(f'sum col_tolens {sum(col_token)}')
    return sum(col_token)